'use strict'
const os = require('os');
const mPath = require('path');
const mFs = require('fs');

let oldValue = 0;
let randoms = [];

///////////////////////////////////////////////////////////////

// Constructor
let UTIL = function() {
	this.randoms = randoms;
};

/********************************************
 * ファイルの存在確認
 *  指定のファイル、フォルダの存在をチェックする。
 *  あればTrue,なければFalseを返す
 ********************************************/
UTIL.prototype.existenceCheck = function(chkPath) {
	try {
		mFs.statSync(chkPath);
		return true;
	} catch(err) {
		return false;
	}
};

/********************************************
 * ファイル書き込み
 *  指定のファイル、フォルダを書き込む。
 *  成功であればTrue,でなければFalseを返す
 ********************************************/
UTIL.prototype.writeFile = function(filePath, writeString) {
	return new Promise(function(resolve){
		mFs.writeFile(filePath, writeString, function (err) {
			if(err){
				resolve(false);
			}else{
				resolve(true);
			}
		});
	});
};

/*********************************************************
* Wait function
*    waitTime : mSec
*********************************************************/
UTIL.prototype.waitFunc = function(waitTime) {
	return new Promise(
		function(resolve){
			setTimeout(function(){
					resolve(true);
				}, waitTime);
		}
	);
};

/*********************************************************
* Ip function
*********************************************************/
UTIL.prototype.getLocalAddress = function() {
	return new Promise(function(resolve){
		let ifacesObj = {}
		ifacesObj.ipv4 = [];
		ifacesObj.ipv6 = [];
		let interfaces = os.networkInterfaces();
		let ipString = "";

		for (let dev in interfaces) {
			interfaces[dev].forEach(function(details){
				if (!details.internal){
					switch(details.family){
						case "IPv4":
							ifacesObj.ipv4.push({name:dev, address:details.address});
							break;
						case "IPv6":
							ifacesObj.ipv6.push({name:dev, address:details.address})
							break;
						default:
							ipString = "0.0.0.0";
							break;
					}
				}
			});
		}
		try{
			if(ifacesObj.ipv4[0].address === undefined){
				ipString = "0.0.0.0";
			}else{
				ipString = ifacesObj.ipv4[0].address;
			}
		}catch( e ){
			// 例外エラーが起きた時に実行する処理
			console.log("例外エラーが発生しました");
			console.log(e.message);
			ipString = "0.0.0.0";
		}
		resolve(ipString);
	});
};

/********************************************
 * 現在日時取得 ISO8601
 *  yyyy-mm-dd
 ********************************************/
UTIL.prototype.getNowDate = function() {
	let dateObjTmp = new Date();
	let timezoneOffset = (dateObjTmp.getTimezoneOffset() / 60) * -1;

	let ISO8601Date = "";
		ISO8601Date += dateObjTmp.getFullYear() + "-";
		ISO8601Date += UTIL.prototype.zeroPadding2Dec(dateObjTmp.getMonth() + 1, 2) + "-";
		ISO8601Date += UTIL.prototype.zeroPadding2Dec(dateObjTmp.getDate(), 2);
	return ISO8601Date;
};

/********************************************
 * 現在時刻取得 ISO8601
 *  yyyy-mm-ddThh:mm:ss+09:00
 *  yyyy-mm-ddThh:mm:ssZ
 ********************************************/
UTIL.prototype.getNowDateTime = function() {
	let dateObjTmp = new Date();
	let timezoneOffset = (dateObjTmp.getTimezoneOffset() / 60) * -1;

	let ISO8601Date = "";
		ISO8601Date += dateObjTmp.getFullYear() + "-";
		ISO8601Date += UTIL.prototype.zeroPadding2Dec(dateObjTmp.getMonth() + 1, 2) + "-";
		ISO8601Date += UTIL.prototype.zeroPadding2Dec(dateObjTmp.getDate(), 2);
		ISO8601Date += "T";
		ISO8601Date += UTIL.prototype.zeroPadding2Dec(dateObjTmp.getHours(), 2) + ":";
		ISO8601Date += UTIL.prototype.zeroPadding2Dec(dateObjTmp.getMinutes(), 2) + ":";
		ISO8601Date += UTIL.prototype.zeroPadding2Dec(dateObjTmp.getSeconds(), 2);
		if(timezoneOffset === 0){
		    ISO8601Date += "Z";
		}else{
		    ISO8601Date += "+" + UTIL.prototype.zeroPadding2Dec(timezoneOffset, 2) + ":00";
		}
	return ISO8601Date;
};
/********************************************
 * 指定時刻取得 ISO8601
 *  yyyy-mm-ddThh:mm:ss+09:00
 *  yyyy-mm-ddThh:mm:ssZ
 ********************************************/
UTIL.prototype.calNowDateTime = function(mSec) {
	let dateObjTmp = new Date(mSec);
	let timezoneOffset = (dateObjTmp.getTimezoneOffset() / 60) * -1;
	let ISO8601Date = "";
		ISO8601Date += dateObjTmp.getFullYear() + "-";
		ISO8601Date += UTIL.prototype.zeroPadding2Dec(dateObjTmp.getMonth() + 1, 2) + "-";
		ISO8601Date += UTIL.prototype.zeroPadding2Dec(dateObjTmp.getDate(), 2);
		ISO8601Date += "T";
		ISO8601Date += UTIL.prototype.zeroPadding2Dec(dateObjTmp.getHours(), 2) + ":";
		ISO8601Date += UTIL.prototype.zeroPadding2Dec(dateObjTmp.getMinutes(), 2) + ":";
		ISO8601Date += UTIL.prototype.zeroPadding2Dec(dateObjTmp.getSeconds(), 2);
		if(timezoneOffset === 0){
		    ISO8601Date += "Z";
		}else{
		    ISO8601Date += "+" + UTIL.prototype.zeroPadding2Dec(timezoneOffset, 2) + ":00";
		}
	return ISO8601Date;
};
/********************************************
 * 指定時刻取得 ISO8601
 *  yyyy-mm-ddThh:mm:ss+09:00
 *  yyyy-mm-ddThh:mm:ssZ
 * timezoneOffset:秒指定
 ********************************************/
UTIL.prototype.calNowDateTimeTimeZone = function(mSec, timezoneOffset) {
	let dateObjTmp = new Date(mSec);
	timezoneOffset = (timezoneOffset / 3600);
	let ISO8601Date = "";
		ISO8601Date += dateObjTmp.getFullYear() + "-";
		ISO8601Date += UTIL.prototype.zeroPadding2Dec(dateObjTmp.getMonth() + 1, 2) + "-";
		ISO8601Date += UTIL.prototype.zeroPadding2Dec(dateObjTmp.getDate(), 2);
		ISO8601Date += "T";
		ISO8601Date += UTIL.prototype.zeroPadding2Dec(dateObjTmp.getHours(), 2) + ":";
		ISO8601Date += UTIL.prototype.zeroPadding2Dec(dateObjTmp.getMinutes(), 2) + ":";
		ISO8601Date += UTIL.prototype.zeroPadding2Dec(dateObjTmp.getSeconds(), 2);
		if(timezoneOffset === 0){
		    ISO8601Date += "Z";
		}else{
		    ISO8601Date += "+" + UTIL.prototype.zeroPadding2Dec(timezoneOffset, 2) + ":00";
		}
	return ISO8601Date;
};
/********************************************
 * 指定時刻取得 ISO8601
 *  yyyymmdd
 ********************************************/
UTIL.prototype.cal_YYYYMMDD_String = function(mSec, timezoneOffset) {
	let dateObjTmp = new Date(mSec);
	timezoneOffset = (timezoneOffset / 3600);
	let ISO8601Date = "";
		ISO8601Date += dateObjTmp.getFullYear();
		ISO8601Date += UTIL.prototype.zeroPadding2Dec(dateObjTmp.getMonth() + 1, 2);
		ISO8601Date += UTIL.prototype.zeroPadding2Dec(dateObjTmp.getDate(), 2);
	return ISO8601Date;
};

/********************************************
 * ISO8601からmSec取得
 *  yyyy-mm-ddThh:mm:ss+09:00
 *  yyyy-mm-ddThh:mm:ssZ
 ********************************************/
UTIL.prototype.isoDateString2milliSeconds = function(dateString) {
	let mSecTemp = new Date(dateString);
	let result = mSecTemp.getTime();
	return result;
};

/********************************************
 * nowからmSec取得
 *  yyyy-mm-ddThh:mm:ss+09:00
 *  yyyy-mm-ddThh:mm:ssZ
 ********************************************/
UTIL.prototype.getNnowMilliSeconds = function() {
	let mowTime = new Date();
	let result = mowTime.getTime();
	return result;
};

/*********************************************************
* LEN桁に0埋め調整[Decで出力]
* NUM=値 LEN=桁数
*********************************************************/
UTIL.prototype.zeroPadding2Dec = function(NUM, LEN) {
	return ( Array(LEN).join('0') + NUM ).slice( -LEN );
};
/*********************************************************
* LEN桁に0埋め調整[Hexで出力]
* NUM=値 LEN=桁数
*********************************************************/
UTIL.prototype.zeroPadding2Hex = function(NUM, LEN) {
	let HexNumStr = NUM.toString(16);
	HexNumStr = HexNumStr.toLowerCase();
	return ( Array(LEN).join('0') + HexNumStr ).slice( -LEN );
};
/*********************************************************
* UTF8文字列をバイナリに変換
* text=文字列
*********************************************************/
UTIL.prototype.UTF8toBinary = function(text) {
	let bytes1 = UTIL.prototype.UTF8String2BbyteAry(text);
	let hex_str1 = UTIL.prototype.bytesAry2HexString(bytes1);
	return hex_str1;
};
/*********************************************************
* バイト配列を16進文字列に変換
* bytesAry=バイト配列
*********************************************************/
UTIL.prototype.bytesAry2HexString = function(bytesAry) {
	let result = "";
	for(let i = 0; i < bytesAry.length; i++) {
		// バイト値を16進文字列に変換
		let digits = bytesAry[i].toString(16);
		if (bytesAry[i] < 16){
			result += ('0' + digits);
		}else{
			result += digits;
		}
	}
	return result;
};

/*********************************************************
* ByteLength function
*********************************************************/
UTIL.prototype.bytesLength = function(str) {
	return(encodeURIComponent(str).replace(/%../g,"x").length);
};
/*********************************************************
* 文字列をUTF8のバイト配列に変換
* text=文字列
*********************************************************/
UTIL.prototype.UTF8String2BbyteAry = function(text) {
	let result = [];
	if(text == null){
		return result;
	}else{
		for(let i = 0; i < text.length; i++){
			let c = text.charCodeAt(i);
			if (c <= 0x7f) {
				result.push(c);
			} else if (c <= 0x07ff) {
				result.push(((c >> 6) & 0x1F) | 0xC0);
				result.push((c & 0x3F) | 0x80);
			} else {
				result.push(((c >> 12) & 0x0F) | 0xE0);
				result.push(((c >> 6) & 0x3F) | 0x80);
				result.push((c & 0x3F) | 0x80);
			}
		}
		return result;
	}
};

/*********************************************************
* split function
* 指定の文字数で区切って配列で返す
*********************************************************/
UTIL.prototype.splitByLength = function(str, length) {
	let resultAry = [];
	if (!str || !length || length < 1) {
		return resultAry;
	}
	let index = 0;
	let start = index;
	let end = start + length;
	while (start < str.length) {
		resultAry[index] = str.substring(start, end);
		index++;
		start = end;
		end = start + length;
	}
	return resultAry;
};

/*********************************************************
* split function hex
* 指定の文字数で区切ってhexで配列で返す
*********************************************************/
UTIL.prototype.splitByLengthForHex = function(str, length) {
	let resultArr = [];
	if (!str || !length || length < 1) {
		resultArr[0] = str;
		return parseInt(resultArr[0], 16);
	}
	let index = 0;
	let start = index;
	let end = start + length;
	while (start < str.length) {
		resultArr[index] = parseInt(str.substring(start, end), 16);
		index++;
		start = end;
		end = start + length;
	}
	return resultArr;
};

/*********************************************************
* ary2Strings function
* 配列の文字列を連結してで返す
*********************************************************/
UTIL.prototype.ary2Strings = function(strAry) {
	let stringTmp = "";
	for(let i=0; i<strAry.length; i++){
		stringTmp += strAry[i];
	}
	return stringTmp;
};

/********************************************
 * 乱数取得
 *  0-255の乱数を生成する。
 * 256回重ならない事を保証(256以後はランダムにならないことに注意)
 ********************************************/
UTIL.prototype.makeIdRandomNumber = function() {
	let randomNum = 0;
	const min   = 0;
	const max   = 256;
	const limit = max;

	// 最大履歴数超えていないか？
	if(this.randoms.length >= limit){
		// 一番古いのを削除して詰めなおす
		randomNum = this.randoms[0];
		this.randoms.splice(0, 1);
		this.randoms.push(randomNum);
		return randomNum;
	}else{
		for(;;){
			let flg = false;
			// min以上 max未満の整数値のみの乱数
			randomNum = Math.floor(Math.random() * (max - min)) + min;
			for(let i=0; i<this.randoms.length; i++){
				if(this.randoms[i] === randomNum){
					// 過去に選出済み
					flg = true;
					break;
				}
			}
			if(flg === true){
				// 再度実行が必要
			}else{
				// 最大履歴数超えていないか(安全策)？
				if(this.randoms.length >= limit){
					// 一番古いのを削除して詰めなおす
					randomNum = this.randoms[0];
					this.randoms.splice(0, 1);
					this.randoms.push(randomNum);
					return randomNum;
				}else{
					// 最新のものを詰める
					this.randoms.push(randomNum);
					return randomNum;
				}
			}
		}
	}
};
/********************************************
 * シーケンシャル数取得
 *  0-255の256回重ならない事を保証する数値作成
 *  (+1されていく0→255→0→の連続数値・・・・）
 ********************************************/
UTIL.prototype.makeIdSequentialNumber = function() {
	if(oldValue >= 255){
		oldValue = 0;
	}else{
		oldValue++;
	}
	return oldValue;
};

/*********************************************************
 * 任意の桁で四捨五入する関数
 * @param {number} value 四捨五入する数値
 * @param {number} base どの桁で四捨五入するか（10→10の位、0.1→小数第１位）
 * @return {number} 四捨五入した値
*********************************************************/
UTIL.prototype.orgRound = function(value, base) {
	return Math.round(value * base) / base;
};

/***************************************************
 * Hex情報の数値化
 * 指定の型の数値にHex文字情報を変換する
 * 
 **************************************************/
UTIL.prototype.HexStrToInt8Num = function(hexStrChar){
	return Int8Array.from([parseInt(hexStrChar,16)])[0];
};
UTIL.prototype.HexStrToUint8Num = function(hexStrUchar){
	return Uint8Array.from([parseInt(hexStrUchar,16)])[0];
};
UTIL.prototype.HexStrToInt16Num = function(hexStrShort){
	return Int16Array.from([parseInt(hexStrShort,16)])[0];
};
UTIL.prototype.HexStrToUint16Num = function(hexStrUshort){
	return Uint16Array.from([parseInt(hexStrUshort,16)])[0];
};
UTIL.prototype.HexStrToInt32Num = function(hexStrInt){
	return Int32Array.from([parseInt(hexStrInt,16)])[0];
};
UTIL.prototype.HexStrToUint32Num = function(hexStrUint){
	return Uint32Array.from([parseInt(hexStrUint,16)])[0];
};
UTIL.prototype.HexStrToLong32Num = function(hexStrLong){
	return Int32Array.from([parseInt(hexStrLong,16)])[0];
};
UTIL.prototype.HexStrToUlong32Num = function(hexStrUlong){
	return Uint32Array.from([parseInt(hexStrUlong,16)])[0];
};
UTIL.prototype.FloatHexStrToDecNum = function(hexStrFloat){
	// Input Hex:Big
	let buf = new Buffer.from(hexStrFloat, "hex");
	let number = buf.readFloatBE(0);
	return number;
};

/*********************************************************
* 小数点以下の桁数を揃える
* NUM=値 LEN=桁数
*********************************************************/
UTIL.prototype.roundFloat = function(NUM, LEN) {
	let lenNum = 1;
	for(let i=0; i<LEN; i++){
		lenNum = lenNum * 10;
	}
	NUM *= lenNum;
	NUM  = Math.round(NUM);
	NUM /= lenNum
	return NUM;
};

/*********************************************************
* ログ出力処理
* logLevelがconfig.json内のDEBUG_LEVEL以下だと出力される
* ※DEBUG_LEVELが0だと何も表示しない
*********************************************************/
UTIL.prototype.debugLog = function(logLevel, configLeve, myName, logString){
	if(configLeve !== 0){
		if(configLeve >= logLevel){
			console.log("[[%s][%s]]:" + logString, UTIL.prototype.getNowDateTime(), myName);
		}
	}
};

/*********************************************************
* HGW毎のユニークIDの取得
*  HGWのUUIDを取得してくる
*********************************************************/
UTIL.prototype.uniqueIdGene = function(){
	const UUID_PATH_V1= __dirname + "/../../../UUID";
	const UUID_PATH_V2= __dirname + "/../../UUID";

	// node-symphonyのUUID ファイルの存在をチェック
	if(mFs.existsSync(UUID_PATH_V2)){
		// UUID ファイルから UUID を読み取る
		let fdata = mFs.readFileSync(UUID_PATH_V2, 'utf8');

		// UUID が取り出せたら終了
		let m = fdata.match(/^([a-f\d]{8}\-[a-f\d]{4}\-[a-f\d]{4}\-[a-f\d]{4}\-[a-f\d]{12})/);
		if (m && m[1]) {
			return "-" + m[1];
		}else{
			return "";
		}
	}else{
		// node-gotapi-symphony-agentのUUID ファイルの存在をチェック
		if(mFs.existsSync(UUID_PATH_V1)){
			// UUID ファイルから UUID を読み取る
			let fdata = mFs.readFileSync(UUID_PATH_V1, 'utf8');

			// UUID が取り出せたら終了
			let m = fdata.match(/^([a-f\d]{8}\-[a-f\d]{4}\-[a-f\d]{4}\-[a-f\d]{4}\-[a-f\d]{12})/);
			if (m && m[1]) {
				return "-" + m[1];
			}else{
				return "";
			}
		}else{
			return "";
		}
	}
};

/*********************************************************
* バリデーション
*  BLE ID
*  ex)aabbccddeeff, aa:bb:cc:dd:ee:ff, aa-bb-cc-dd-ee-ff, aa bb cc dd ee ff",
*********************************************************/
UTIL.prototype.bleIdChk = function(bleId){
	const regexp_0 = /^[0-9a-fA-F]{12}/;
	const regexp_1 = /^[0-9a-fA-F]{2}(:[0-9a-fA-F]{2}){5}$/i;
	const regexp_2 = /^[0-9a-fA-F]{2}(-[0-9a-fA-F]{2}){5}$/i;
	const regexp_3 = /^[0-9a-fA-F]{2}( [0-9a-fA-F]{2}){5}$/i;
	if(regexp_0.test(bleId)){
		return true;
	}else if(regexp_1.test(bleId)){
		return true;
	}else if(regexp_2.test(bleId)){
		return true;
	}else if(regexp_3.test(bleId)){
		return true;
	}else{
		return false;
	}
};
UTIL.prototype.bleIdConversion = function(bleId){
	let idAry = UTIL.prototype.splitByLength(bleId, 2);
	let idStr = idAry[0] + ":" + idAry[1] + ":" + idAry[2] + ":" + idAry[3] + ":" + idAry[4] + ":" + idAry[5];
	idStr = idStr.toUpperCase();
	return idStr;
};

/*********************************************************
* バリデーション
*  ストリングチェック
*********************************************************/
UTIL.prototype.isString = function(value){
	if( (typeof value === "string") || (value instanceof String) ){
		return true;
	}else{
		return false;
	}
};

module.exports = UTIL;
